﻿/*
@@@START_XML@@@
<?xml version="1.0" encoding="UTF-8"?>
<ScriptInfo xmlns:dc="http://purl.org/dc/elements/1.1/" xml:lang="en_US">
<dc:title>Alpine Blue - Set Date Shot/Created based on Name</dc:title>
<dc:description>This script allows users to set the dateShot and DateCreated metadata properties based on the date imbeded in the name. 
To use: Right-click on a thumbnail which includes the pattern YYYY-MM-DD hh.mm.ss in the name, select "Set Date Shot/Created based on Name" from the context menu.</dc:description>
<dc:source>mr.roger.wilson@gmail.com</dc:soruce>
</ScriptInfo>
@@@END_XML@@@
*/


function AlpineBlueSetDateShot()
{
	this.menuID = "AlpineBlueSetDateShotContextMenu";
}

AlpineBlueSetDateShot.prototype.run = function()
{
	if(!this.canRun())
	{
		return false;
	}

	// Create the menu element
	var ctxCommand = new MenuElement("command", "Set Date Shot/Created based on Name", "at the end of Thumbnail", this.menuID);
	
	// What to do when the menu item is selected
	ctxCommand.onSelect = function(m)
	{
		// Load the XMP Script library
		if( xmpLib == undefined ) 
		{
			if( Folder.fs == "Windows" )
			{
				var pathToLib = Folder.startup.fsName + "/AdobeXMPScript.dll";
			} 
			else 
			{
				var pathToLib = Folder.startup.fsName + "/AdobeXMPScript.framework";
			}
		
			var libfile = new File( pathToLib );
			var xmpLib = new ExternalObject("lib:" + pathToLib );
		}

		var selLength = app.document.selectionLength;
		var selSnapshot = app.document.selections;
		for(var index = 0; index < selLength; index++)
		{
			var thumbnail = selSnapshot[index];
			
			// skip any containers
			if(thumbnail.container)
			{
				continue;
			}

			// update the status line
			var name = thumbnail.name;
			var statusMessage = "Updating " + name + " (" + (index+1).toString() + " of " + selLength.toString() + ")";
			app.document.status = statusMessage;
			
			// parse out the date from the file name YYYY-MM-DD hh.mm.ss
			var start = name.search(/\d{4}-\d{2}-\d{2}\s\d{2}.\d{2}.\d{2}/);
			if ( start != -1 )
			{
				var year = parseInt(name.substring(start + 0, start + 0 + 4));
				var month = parseInt(name.substring(start + 5, start + 5 + 2));
				var day = parseInt(name.substring(start + 8, start + 8 + 2));
				var hour = parseInt(name.substring(start + 11, start + 11 + 2));
				var minute = parseInt(name.substring(start + 14, start + 14 + 2));
				var second = parseInt(name.substring(start + 17, start + 17 + 2));
				var dateShot = new Date(year, month-1, day, hour, minute, second);
				var xmpDateShot = new XMPDateTime(dateShot);
				
				// get the current meta data
				if(thumbnail.hasMetadata)
				{
					// Get the metadata object - wait for valid values
					var md = thumbnail.synchronousMetadata;
					
					// Get the XMP packet as a string and create the XMPMeta object
					var xmp = new XMPMeta(md.serialize());
					
					// Set the Created Dtae and Shot Date fields
					xmp.setProperty("http://ns.adobe.com/photoshop/1.0/", "DateCreated", xmpDateShot, XMPConst.XMPDATE);
					xmp.setProperty("http://ns.adobe.com/xmp/1.0/DynamicMedia/", "shotDate", xmpDateShot, XMPConst.XMPDATE);
					
					// Write the packet back to the selected file
					var updatedPacket = xmp.serialize(XMPConst.SERIALIZE_OMIT_PACKET_WRAPPER | XMPConst.SERIALIZE_USE_COMPACT_FORMAT);

					// Uncomment to see the XMP packet in XML form
					// $.writeln(updatedPacket);
					thumbnail.metadata = new Metadata(updatedPacket);
				} // end hasMetadata
			
			} // end match
		} // end for
	} // end onSelect

	ctxCommand.onDisplay = function()
	{
		try
		{
			 enabled = false;
			
			var selLength = app.document.selectionLength;
			var selSnapshot = app.document.selections;
			for(var index = 0; index < selLength; index++)
			{
				var thumbnail = selSnapshot[index];
				var start = thumbnail.name.search(/\d{4}-\d{2}-\d{2}\s\d{2}.\d{2}.\d{2}/);
				if ( start != -1 )
				{
					enabled = true;
					break;
				}
			}
		
			ctxCommand.enabled = enabled;
		}
		catch(error)
		{
			$.writeln(error);
		}
	} // end onDisplay

	return true;
}

AlpineBlueSetDateShot.prototype.canRun = function()
 {
	// Must be running in Bridge & have a selection
	if(BridgeTalk.appName == "bridge") 
	{
		// Stop the menu element from being added again if the snippet has already run
		if(MenuElement.find(this.menuID))
		{
			$.writeln("ERROR:: Menu element " + this.menuID + " already exists!\nRestart Bridge to run this snippet again.");
			return false;
		}
		return true;
	}

	$.writeln("ERROR:: Cannot run AlpineBlueSetDateShot");
	$.writeln("Needs to run in Bridge, \nwith a selection of a file.");
	return false;
}

/**
 "main program": construct an anonymous instance and run it
  as long as we are not unit-testing this snippet.
*/
if(typeof(AlpineBlueSetDateShot_unitTest ) == "undefined") {
	new AlpineBlueSetDateShot().run();
}
